/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 7: Naive estimation of individual treatment effects
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 7.1: Creating and printing a dataset holding the naive individual treatment effects
*****************************************************************************************************************/

proc means data=NORMAL.ddata alpha=0.05 noprint;
	by Patient;
	var dY;
	output
		out=NORMAL.sddata( drop=_TYPE_ rename=(_FREQ_=nobs) )
		mean=meanY
		stderr=stdY
		t=tY
		probt=probtY
		lclm=lowY
		uclm=uppY;
run;

proc print data=NORMAL.sddata noobs label;
	label
		Patient='Patient'
		nobs='N Obs'
		meanY='Mean'
		stdY='Std Error'
		tY="t Value"
		probtY="Pr > |t|"
		lowY='Lower 95% CL for Mean'
		uppY='Upper 95% CL for Mean';
	title 'Naive individual treatment effects for balanced data';
run;

/*****************************************************************************************************************
Program 7.2: Computing the extreme values of naive individual treatment effects
*****************************************************************************************************************/

title "Extreme values of naive individual treatment effects";
proc sql;
	select	Patient,
			meanY as Extreme
	from NORMAL.sddata
	having meanY eq min(meanY)
	or meanY eq max(meanY);
quit;

/*****************************************************************************************************************
Program 7.3: Computing summary statistics of naive individual treatment effects
*****************************************************************************************************************/

title "Summary statistics of naive individual treatment effects";
proc summary
		data=NORMAL.sddata
		print
		maxdec=2
		min q1 mean median q3 max;
	var meanY;
run;

/*****************************************************************************************************************
Program 7.4: Computing naive individual treatment effects
*****************************************************************************************************************/

proc means
		data=NORMAL.ddata_unbalanced
		alpha=0.05
		maxdec=4
		mean stderr t probt clm;
	class Patient;
	var dY;
	title 'Naive individual treatment effects for unbalanced data';
	ods noproctitle; * remove procedure title from output ;
run;

title; * clear all titles ;
