/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 10: Analysis of variance
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 10.1: Performing an analysis of variance using proc anova
*****************************************************************************************************************/

title "ANOVA Cycle within Patient as block structure - proc anova";
proc anova data=NORMAL.ndata plots=none;
	class Treatment Patient Cycle;
	model Y = Treatment Patient Cycle(Patient);
	means Treatment;
run;
quit;

/*****************************************************************************************************************
Program 10.2: Performing an analysis of variance using proc glm
*****************************************************************************************************************/

title "ANOVA Cycle within Patient as block structure - proc glm";
proc glm data=NORMAL.ndata_unbalanced plots=none;
	class Treatment Patient Cycle;
	model Y = Treatment Patient Cycle(Patient);
	means Treatment;
run;
quit;

/*****************************************************************************************************************
Program 10.3: Performing an analysis of variance with interactions using proc glm
*****************************************************************************************************************/

title "ANOVA Treatment by Patient and Cycle by Patient interaction - proc glm";
proc glm data=NORMAL.ndata plots=none;
	class Treatment Patient Cycle;
	model Y = Treatment Patient Treatment*Patient Cycle*Patient;
	means Treatment;
run;
quit;

title; * clear all titles ;
